use crate::co::*;

const_values! { HRESULT;
	=>
	VFW_E_INVALIDMEDIATYPE 0x8004_0200
	VFW_E_INVALIDSUBTYPE 0x8004_0201
	VFW_E_NEED_OWNER 0x8004_0202
	VFW_E_ENUM_OUT_OF_SYNC 0x8004_0203
	VFW_E_ALREADY_CONNECTED 0x8004_0204
	VFW_E_FILTER_ACTIVE 0x8004_0205
	VFW_E_NO_TYPES 0x8004_0206
	VFW_E_NO_ACCEPTABLE_TYPES 0x8004_0207
	VFW_E_INVALID_DIRECTION 0x8004_0208
	VFW_E_NOT_CONNECTED 0x8004_0209
	VFW_E_NO_ALLOCATOR 0x8004_020a
	VFW_E_RUNTIME_ERROR 0x8004_020b
	VFW_E_BUFFER_NOTSET 0x8004_020c
	VFW_E_BUFFER_OVERFLOW 0x8004_020d
	VFW_E_BADALIGN 0x8004_020e
	VFW_E_ALREADY_COMMITTED 0x8004_020f
	VFW_E_BUFFERS_OUTSTANDING 0x8004_0210
	VFW_E_NOT_COMMITTED 0x8004_0211
	VFW_E_SIZENOTSET 0x8004_0212
	VFW_E_NO_CLOCK 0x8004_0213
	VFW_E_NO_SINK 0x8004_0214
	VFW_E_NO_INTERFACE 0x8004_0215
	VFW_E_NOT_FOUND 0x8004_0216
	VFW_E_CANNOT_CONNECT 0x8004_0217
	VFW_E_CANNOT_RENDER 0x8004_0218
	VFW_E_CHANGING_FORMAT 0x8004_0219
	VFW_E_NO_COLOR_KEY_SET 0x8004_021a
	VFW_E_NOT_OVERLAY_CONNECTION 0x8004_021b
	VFW_E_NOT_SAMPLE_CONNECTION 0x8004_021c
	VFW_E_PALETTE_SET 0x8004_021d
	VFW_E_COLOR_KEY_SET 0x8004_021e
	VFW_E_NO_COLOR_KEY_FOUND 0x8004_021f
	VFW_E_NO_PALETTE_AVAILABLE 0x8004_0220
	VFW_E_NO_DISPLAY_PALETTE 0x8004_0221
	VFW_E_TOO_MANY_COLORS 0x8004_0222
	VFW_E_STATE_CHANGED 0x8004_0223
	VFW_E_NOT_STOPPED 0x8004_0224
	VFW_E_NOT_PAUSED 0x8004_0225
	VFW_E_NOT_RUNNING 0x8004_0226
	VFW_E_WRONG_STATE 0x8004_0227
	VFW_E_START_TIME_AFTER_END 0x8004_0228
	VFW_E_INVALID_RECT 0x8004_0229
	VFW_E_TYPE_NOT_ACCEPTED 0x8004_022a
	VFW_E_SAMPLE_REJECTED 0x8004_022b
	VFW_E_SAMPLE_REJECTED_EOS 0x8004_022c
	VFW_E_DUPLICATE_NAME 0x8004_022d
	VFW_S_DUPLICATE_NAME 0x0004_022d
	VFW_E_TIMEOUT 0x8004_022e
	VFW_E_INVALID_FILE_FORMAT 0x8004_022f
	VFW_E_ENUM_OUT_OF_RANGE 0x8004_0230
	VFW_E_CIRCULAR_GRAPH 0x8004_0231
	VFW_E_NOT_ALLOWED_TO_SAVE 0x8004_0232
	VFW_E_TIME_ALREADY_PASSED 0x8004_0233
	VFW_E_ALREADY_CANCELLED 0x8004_0234
	VFW_E_CORRUPT_GRAPH_FILE 0x8004_0235
	VFW_E_ADVISE_ALREADY_SET 0x8004_0236
	VFW_S_STATE_INTERMEDIATE 0x0004_0237
	VFW_E_NO_MODEX_AVAILABLE 0x8004_0238
	VFW_E_NO_ADVISE_SET 0x8004_0239
	VFW_E_NO_FULLSCREEN 0x8004_023a
	VFW_E_IN_FULLSCREEN_MODE 0x8004_023b
	VFW_E_UNKNOWN_FILE_TYPE 0x8004_0240
	VFW_E_CANNOT_LOAD_SOURCE_FILTER 0x8004_0241
	VFW_S_PARTIAL_RENDER 0x0004_0242
	VFW_E_FILE_TOO_SHORT 0x8004_0243
	VFW_E_INVALID_FILE_VERSION 0x8004_0244
	VFW_S_SOME_DATA_IGNORED 0x0004_0245
	VFW_S_CONNECTIONS_DEFERRED 0x0004_0246
	VFW_E_INVALID_CLSID 0x8004_0247
	VFW_E_INVALID_MEDIA_TYPE 0x8004_0248
	VFW_E_BAD_KEY 0x8004_03f2
	VFW_S_NO_MORE_ITEMS 0x0004_0103
	VFW_E_SAMPLE_TIME_NOT_SET 0x8004_0249
	VFW_S_RESOURCE_NOT_NEEDED 0x0004_0250
	VFW_E_MEDIA_TIME_NOT_SET 0x8004_0251
	VFW_E_NO_TIME_FORMAT_SET 0x8004_0252
	VFW_E_MONO_AUDIO_HW 0x8004_0253
	VFW_S_MEDIA_TYPE_IGNORED 0x0004_0254
	VFW_E_NO_DECOMPRESSOR 0x8004_0255
	VFW_E_NO_AUDIO_HARDWARE 0x8004_0256
	VFW_S_VIDEO_NOT_RENDERED 0x0004_0257
	VFW_S_AUDIO_NOT_RENDERED 0x0004_0258
	VFW_E_RPZA 0x8004_0259
	VFW_S_RPZA 0x0004_025a
	VFW_E_PROCESSOR_NOT_SUITABLE 0x8004_025b
	VFW_E_UNSUPPORTED_AUDIO 0x8004_025c
	VFW_E_UNSUPPORTED_VIDEO 0x8004_025d
	VFW_E_MPEG_NOT_CONSTRAINED 0x8004_025e
	VFW_E_NOT_IN_GRAPH 0x8004_025f
	VFW_S_ESTIMATED 0x0004_0260
	VFW_E_NO_TIME_FORMAT 0x8004_0261
	VFW_E_READ_ONLY 0x8004_0262
	VFW_S_RESERVED 0x0004_0263
	VFW_E_BUFFER_UNDERFLOW 0x8004_0264
	VFW_E_UNSUPPORTED_STREAM 0x8004_0265
	VFW_E_NO_TRANSPORT 0x8004_0266
	VFW_S_STREAM_OFF 0x0004_0267
	VFW_S_CANT_CUE 0x0004_0268
	VFW_E_BAD_VIDEOCD 0x8004_0269
	VFW_S_NO_STOP_TIME 0x0004_0270
	VFW_E_OUT_OF_VIDEO_MEMORY 0x8004_0271
	VFW_E_VP_NEGOTIATION_FAILED 0x8004_0272
	VFW_E_DDRAW_CAPS_NOT_SUITABLE 0x8004_0273
	VFW_E_NO_VP_HARDWARE 0x8004_0274
	VFW_E_NO_CAPTURE_HARDWARE 0x8004_0275
	VFW_E_DVD_OPERATION_INHIBITED 0x8004_0276
	VFW_E_DVD_INVALIDDOMAIN 0x8004_0277
	VFW_E_DVD_NO_BUTTON 0x8004_0278
	VFW_E_DVD_GRAPHNOTREADY 0x8004_0279
	VFW_E_DVD_RENDERFAIL 0x8004_027a
	VFW_E_DVD_DECNOTENOUGH 0x8004_027b
	VFW_E_DDRAW_VERSION_NOT_SUITABLE 0x8004_027c
	VFW_E_COPYPROT_FAILED 0x8004_027d
	VFW_S_NOPREVIEWPIN 0x0004_027e
	VFW_E_TIME_EXPIRED 0x8004_027f
	VFW_S_DVD_NON_ONE_SEQUENTIAL 0x0004_0280
	VFW_E_DVD_WRONG_SPEED 0x8004_0281
	VFW_E_DVD_MENU_DOES_NOT_EXIST 0x8004_0282
	VFW_E_DVD_CMD_CANCELLED 0x8004_0283
	VFW_E_DVD_STATE_WRONG_VERSION 0x8004_0284
	VFW_E_DVD_STATE_CORRUPT 0x8004_0285
	VFW_E_DVD_STATE_WRONG_DISC 0x8004_0286
	VFW_E_DVD_INCOMPATIBLE_REGION 0x8004_0287
	VFW_E_DVD_NO_ATTRIBUTES 0x8004_0288
	VFW_E_DVD_NO_GOUP_PGC 0x8004_0289
	VFW_E_DVD_LOW_PARENTAL_LEVEL 0x8004_028a
	VFW_E_DVD_NOT_IN_KARAOKE_MODE 0x8004_028b
	VFW_S_DVD_CHANNEL_CONTENTS_NOT_AVAILABLE 0x0004_028c
	VFW_S_DVD_NOT_ACCURATE 0x0004_028d
	VFW_E_FRAME_STEP_UNSUPPORTED 0x8004_028e
	VFW_E_DVD_STREAM_DISABLED 0x8004_028f
	VFW_E_DVD_TITLE_UNKNOWN 0x8004_0290
	VFW_E_DVD_INVALID_DISC 0x8004_0291
	VFW_E_DVD_NO_RESUME_INFORMATION 0x8004_0292
	VFW_E_PIN_ALREADY_BLOCKED_ON_THIS_THREAD 0x8004_0293
	VFW_E_PIN_ALREADY_BLOCKED 0x8004_0294
	VFW_E_CERTIFICATION_FAILURE 0x8004_0295
	VFW_E_VMR_NOT_IN_MIXER_MODE 0x8004_0296
	VFW_E_VMR_NO_AP_SUPPLIED 0x8004_0297
	VFW_E_VMR_NO_DEINTERLACE_HW 0x8004_0298
	VFW_E_VMR_NO_PROCAMP_HW 0x8004_0299
	VFW_E_DVD_VMR9_INCOMPATIBLEDEC 0x8004_029a
	VFW_E_NO_COPP_HW 0x8004_029b
	VFW_E_DVD_NONBLOCKING 0x8004_029c
	VFW_E_DVD_TOO_MANY_RENDERERS_IN_FILTER_GRAPH 0x8004_029d
	VFW_E_DVD_NON_EVR_RENDERER_IN_FILTER_GRAPH 0x8004_029e
	VFW_E_DVD_RESOLUTION_ERROR 0x8004_029f
	E_PROP_SET_UNSUPPORTED 0x8007_0492
	E_PROP_ID_UNSUPPORTED 0x8007_0490
	VFW_E_CODECAPI_LINEAR_RANGE 0x8004_0310
	VFW_E_CODECAPI_ENUMERATED 0x8004_0311
	VFW_E_CODECAPI_NO_DEFAULT 0x8004_0313
	VFW_E_CODECAPI_NO_CURRENT_VALUE 0x8004_0314
	VFW_E_DVD_CHAPTER_DOES_NOT_EXIST 0x8004_0315
	VFW_S_DVD_RENDER_STATUS 0x0004_0320
}
